#include "Event.hpp"

Event::Event() throw() : EventHandlers(0),Capacity(0),
    Length(0),BlockSize(32)
{
    EventHandlers = new IDelegate*[BlockSize];
    
    if (EventHandlers != 0)
    {
        Capacity = 32;
        
        for (unsigned x = 0; x < Capacity; ++x)
        {
            EventHandlers[x] = 0;
        }
    }
}

Event::~Event() throw()
{
    if (EventHandlers != 0)
    {
        for (unsigned long x = 0; x < Length; ++x)
        {
            delete EventHandlers[x];
        }
        
        delete [] EventHandlers;
    }
}

bool Event::AddHandler(IDelegate * pDelegate) throw()
{
    if (pDelegate == 0)
    {
        return false;
    }
    
    IObject * pObject = pDelegate->GetObject();
    EventHandler pfnCallback = pDelegate->GetCallback();
    
    bool bExists(false);
    
    for (unsigned long x = 0; x < Length; ++x)
    {
        if (pObject == EventHandlers[x]->GetObject() &&
            pfnCallback == EventHandlers[x]->GetCallback())
        {
            bExists = true;
            break;
        }
    }
    
    if (bExists == false)
    {
        const unsigned long dwNewLength = Length + 1;
        
        if (dwNewLength > Capacity)
        {
            const unsigned long dwAlloc = DetermineCapacity(dwNewLength);
            
            IDelegate ** ppNewData = new IDelegate*[dwAlloc];
            
            if (ppNewData == 0)
            {
                return false;
            }
            
            if (EventHandlers != 0)
            { 
                for (unsigned long x = 0; x < Length; ++x)
                {
                    ppNewData[x] = EventHandlers[x];
                }
                
                delete [] EventHandlers;
            }
            
            EventHandlers = ppNewData;
            Capacity = dwAlloc;
        }
        
        EventHandlers[Length] = pDelegate;
        ++Length;
        
        return true;
    }
    
    return false;
}

bool Event::RemoveHandler(IDelegate * pDelegate) throw()
{
    if (pDelegate == 0)
    {
        return false;
    }
    
    IObject * pObject = pDelegate->GetObject();
    EventHandler pfnCallback = pDelegate->GetCallback();
    
    bool bExists(false);
    unsigned long dwIndex(0);
    
    for (unsigned long x = 0; x < Length; ++x)
    {
        if (pObject == EventHandlers[x]->GetObject() &&
            pfnCallback == EventHandlers[x]->GetCallback())
        {
            dwIndex = x;
            bExists = true;
            break;
        }
    }
    
    if (bExists == true)
    {
        delete EventHandlers[dwIndex];
        
        for (unsigned long i = dwIndex; i < (Length - 1); ++i)
        {
            EventHandlers[i] = EventHandlers[i + 1];
        }
        
        --Length;
        
        return true;
    }
    
    return false;
}

void Event::RemoveAllHandlers() throw()
{
    for (unsigned long x = 0; x < Length; ++x)
    {
        delete EventHandlers[x];
        EventHandlers[x] = 0;
    }
    
    Length = 0;
}

void Event::Raise(IObject * pSender,EventArgs * pArgs) throw()
{
    for (unsigned long x = 0; x < Length; ++x)
    {
        EventHandlers[x]->SetArguments(pSender,pArgs);
        EventHandlers[x]->Invoke();
    }
}

unsigned long Event::DetermineCapacity(unsigned long dwElemCount) throw()
{
    unsigned long dwBlockCount = dwElemCount / BlockSize;
    
    if (dwBlockCount == 0 || (dwElemCount % BlockSize) != 0)
    {
        ++dwBlockCount;
    }
    
    return dwBlockCount * BlockSize;
}
